@@ -126,6 +126,94 @@ GetHierNode(
     return(nn->efnn_node);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * spcHierWriteValue ---
+ *
+ * Special case of spcHierWriteParams() below.  Only check for 'r' or 'c'
+ * parameters which have no parameter name.  Write out the given value,
+ * which in the case of CDL format is written out after the pins but before
+ * everything else, arbitrarily.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+spcHierWriteValue(
+    HierContext *hc,
+    Dev *dev)           /* Dev being output */
+{
+    DevParam *plist;
+
+    plist = efGetDeviceParams(EFDevTypes[dev->dev_type]);
+    while (plist != NULL)
+    {
+	switch (plist->parm_type[0])
+	{
+	    case 'r':
+		if (*(plist->parm_name) == '\0')
+		{
+		    fprintf(esSpiceF, " ");
+		    esSIvalue(esSpiceF, (double)(dev->dev_res));
+		}
+		break;
+	    case 'c':
+		if (*(plist->parm_name) == '\0')
+		{
+		    fprintf(esSpiceF, " ");
+		    esSIvalue(esSpiceF, (double)(dev->dev_cap));
+		}
+		break;
+	}
+	plist = plist->parm_next;
+    }
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * spcHierWriteSubParam ---
+ *
+ * Special case of spcHierWriteParams() below.  Only check for the substrate
+ * parameter, which in the case of CDL format is written out before the
+ * device model, for whatever reason.  Return TRUE if the substrate parameter
+ * was output;  otherwise return FALSE.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+spcHierWriteSubParam(
+    HierContext *hc,
+    Dev *dev)           /* Dev being output */
+{
+    DevParam *plist;
+    bool retvalue = FALSE;
+
+    plist = efGetDeviceParams(EFDevTypes[dev->dev_type]);
+    while (plist != NULL)
+    {
+	switch (plist->parm_type[0])
+	{
+	    case 's':
+		if (dev->dev_subsnode == NULL)
+		    TxError("Error:  Device %s missing substrate node!\n",
+				EFDevTypes[dev->dev_type]);
+		else
+		{
+		    fprintf(esSpiceF, " %s=", plist->parm_name);
+		    spcdevSubstrate(hc->hc_hierName,
+				dev->dev_subsnode->efnode_name->efnn_hier,
+				dev->dev_type, esSpiceF);
+		    retvalue = TRUE;
+		}
+		break;
+	}
+	plist = plist->parm_next;
+    }
+    return retvalue;
+}
 
 /*
  * ----------------------------------------------------------------------------
@@ -145,7 +233,8 @@ spcHierWriteParams(
     float scale,        /* Scale transform for output */
     int l,              /* Device length, in internal units */
     int w,              /* Device width, in internal units */
-    float sdM)          /* Device multiplier */
+    float sdM,          /* Device multiplier */
+    bool subdone)	/* If TRUE, then substrate parameter was already output */
 {
     DevParam *plist, *dparam;
     int parmval;
@@ -342,10 +431,19 @@ spcHierWriteParams(
 		}
 		break;
 	    case 's':
-		fprintf(esSpiceF, " %s=", plist->parm_name);
-		/*EFNode *subnodeFlat =*/ spcdevSubstrate(hc->hc_hierName,
-			dev->dev_subsnode->efnode_name->efnn_hier,
-			dev->dev_type, esSpiceF);
+		if (!subdone)
+		{
+		    if (dev->dev_subsnode == NULL)
+			TxError("Error:  Device %s missing substrate node!\n",
+				EFDevTypes[dev->dev_type]);
+		    else
+		    {
+			fprintf(esSpiceF, " %s=", plist->parm_name);
+			spcdevSubstrate(hc->hc_hierName,
+				dev->dev_subsnode->efnode_name->efnn_hier,
+				dev->dev_type, esSpiceF);
+		    }
+		}
 		break;
 	    case 'x':
 		fprintf(esSpiceF, " %s=", plist->parm_name);
@@ -370,12 +468,18 @@ spcHierWriteParams(
 				* scale * esScale * 1.0E-6);
 		break;
 	    case 'r':
-		fprintf(esSpiceF, " %s=", plist->parm_name);
-		esSIvalue(esSpiceF, (double)(dev->dev_res));
+		if (*(plist->parm_name) != '\0')
+		{
+		    fprintf(esSpiceF, " %s=", plist->parm_name);
+		    esSIvalue(esSpiceF, (double)(dev->dev_res));
+		}
 		break;
 	    case 'c':
-		fprintf(esSpiceF, " %s=", plist->parm_name);
-		esSIvalue(esSpiceF, (double)(dev->dev_cap));
+		if (*(plist->parm_name) != '\0')
+		{
+		    fprintf(esSpiceF, " %s=", plist->parm_name);
+		    esSIvalue(esSpiceF, (double)(dev->dev_cap));
+		}
 		break;
 	}
 	plist = plist->parm_next;
@@ -449,10 +553,16 @@ esOutputHierResistor(
     {
 	fprintf(esSpiceF, " %f", ((double)(dev->dev_res)
 			/ (double)(dscale)) / (double)sdM);
-	spcHierWriteParams(hc, dev, scale, l, w, sdM);
+	spcHierWriteParams(hc, dev, scale, l, w, sdM, FALSE);
     }
     else
     {
+	bool subdone = FALSE;
+
+	spcHierWriteValue(hc, dev);
+	if (esFormat == CDL)
+	    subdone = spcHierWriteSubParam(hc, dev);
+
 	fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
 
 	if (esScale < 0)
@@ -467,7 +577,7 @@ esOutputHierResistor(
 	    fprintf(esSpiceF, " l=");
 	    esSIvalue(esSpiceF, 1.0E-6 * (float)((l * scale * esScale) / dscale));
 	}
-	spcHierWriteParams(hc, dev, scale, l, w, sdM);
+	spcHierWriteParams(hc, dev, scale, l, w, sdM, subdone);
 	if (sdM != 1.0)
 	    fprintf(esSpiceF, " M=%g", sdM);
     }
@@ -584,6 +694,7 @@ spcdevHierVisit(
     float sdM;
     char devchar;
     bool has_model = TRUE;
+    bool subdone = FALSE;
 
     /* If no terminals, or only a gate, can't do much of anything */
     if (dev->dev_nterm <= 1 )
@@ -765,9 +876,11 @@ spcdevHierVisit(
 			source->dterm_node->efnode_name->efnn_hier,
 			"base", esSpiceF);
 
+	    if (esFormat == CDL)
+		subdone = spcHierWriteSubParam(hc, dev);
 	    fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
 	    sdM = getCurDevMult();
-	    spcHierWriteParams(hc, dev, scale, l, w, sdM);
+	    spcHierWriteParams(hc, dev, scale, l, w, sdM, subdone);
 	    break;
 
 	case DEV_MSUBCKT:
@@ -846,7 +959,7 @@ spcdevHierVisit(
 
 	    /* Write all requested parameters to the subcircuit call.	*/
 	    sdM = getCurDevMult();
-	    spcHierWriteParams(hc, dev, scale, l, w, sdM);
+	    spcHierWriteParams(hc, dev, scale, l, w, sdM, FALSE);
 	    if (sdM != 1.0)
 		fprintf(esSpiceF, " M=%g", sdM);
 	    break;
@@ -917,9 +1030,11 @@ spcdevHierVisit(
 		spcdevOutNode(hc->hc_hierName,
 			subnode->efnode_name->efnn_hier,
 			"diode_bot", esSpiceF);
+	    if (esFormat == CDL)
+		subdone = spcHierWriteSubParam(hc, dev);
 	    fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
 	    sdM = getCurDevMult();
-	    spcHierWriteParams(hc, dev, scale, l, w, sdM);
+	    spcHierWriteParams(hc, dev, scale, l, w, sdM, subdone);
 	    break;
 
 	case DEV_NDIODE:
@@ -938,9 +1053,11 @@ spcdevHierVisit(
 	    spcdevOutNode(hc->hc_hierName,
 			gate->dterm_node->efnode_name->efnn_hier,
 			"diode_top", esSpiceF);
+	    if (esFormat == CDL)
+		subdone = spcHierWriteSubParam(hc, dev);
 	    fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
 	    sdM = getCurDevMult();
-	    spcHierWriteParams(hc, dev, scale, l, w, sdM);
+	    spcHierWriteParams(hc, dev, scale, l, w, sdM, subdone);
 	    break;
 
 	case DEV_CAP:
@@ -967,10 +1084,13 @@ spcdevHierVisit(
 	    if (!has_model)
 	    {
 		esSIvalue(esSpiceF, 1.0E-15 * (double)sdM * (double)dev->dev_cap);
-		spcHierWriteParams(hc, dev, scale, l, w, sdM);
+		spcHierWriteParams(hc, dev, scale, l, w, sdM, FALSE);
 	    }
 	    else
 	    {
+		spcHierWriteValue(hc, dev);
+		if (esFormat == CDL)
+		    subdone = spcHierWriteSubParam(hc, dev);
 		fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
 
 		if (esScale < 0)
@@ -984,7 +1104,7 @@ spcdevHierVisit(
 		    fprintf(esSpiceF, " l=");
 		    esSIvalue(esSpiceF, 1.0E-6 * l * scale * esScale);
 		}
-		spcHierWriteParams(hc, dev, scale, l, w, sdM);
+		spcHierWriteParams(hc, dev, scale, l, w, sdM, subdone);
 		if (sdM != 1.0)
 		    fprintf(esSpiceF, " M=%g", sdM);
 	    }
@@ -1014,10 +1134,13 @@ spcdevHierVisit(
 	    if (!has_model)
 	    {
 		esSIvalue(esSpiceF, 1.0E-15 * (double)sdM * (double)dev->dev_cap);
-		spcHierWriteParams(hc, dev, scale, l, w, sdM);
+		spcHierWriteParams(hc, dev, scale, l, w, sdM, FALSE);
 	    }
 	    else
 	    {
+		spcHierWriteValue(hc, dev);
+		if (esFormat == CDL)
+		    subdone = spcHierWriteSubParam(hc, dev);
 		fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
 
 		if (esScale < 0)
@@ -1031,7 +1154,7 @@ spcdevHierVisit(
 		    fprintf(esSpiceF, " l=");
 		    esSIvalue(esSpiceF, 1.0E-6 * l * scale * esScale);
 		}
-		spcHierWriteParams(hc, dev, scale, l, w, sdM);
+		spcHierWriteParams(hc, dev, scale, l, w, sdM, subdone);
 		if (sdM != 1.0)
 		    fprintf(esSpiceF, " M=%g", sdM);
 	    }
@@ -1075,7 +1198,7 @@ spcdevHierVisit(
 		fprintf(esSpiceF, " l=");
 		esSIvalue(esSpiceF, 1.0E-6 * l * scale * esScale);
 	    }
-	    spcHierWriteParams(hc, dev, scale, l, w, sdM);
+	    spcHierWriteParams(hc, dev, scale, l, w, sdM, FALSE);
 	    if (sdM != 1.0)
 		fprintf(esSpiceF, " M=%g", sdM);
 
