@@ -62,13 +62,12 @@ char *EFLayerNames[MAXTYPES] = { "space" };
 int   EFLayerNumNames;
 
 /* Forward declarations */
-Connection *efAllocConn();
-EFNode *efBuildDevNode();
-void efNodeAddName();
-EFNode *efNodeMerge();
+EFNode *efBuildDevNode(Def *def, char *name, bool isSubsNode);
+void efNodeAddName(EFNode *node, HashEntry *he, HierName *hn, bool isNew);
+EFNode *efNodeMerge(EFNode **node1ptr, EFNode **node2ptr);
 
-bool efConnBuildName();
-bool efConnInitSubs();
+bool efConnBuildName(ConnName *cnp, char *name);
+bool efConnInitSubs(Connection *conn, char *nodeName1, char *nodeName2);
 
 extern float locScale;
 
@@ -832,13 +831,13 @@ efBuildDeviceParams(name, argc, argv)
  */
 
 int
-efBuildDevice(def, class, type, r, argc, argv)
-    Def *def;		/* Def to which this connection is to be added */
-    char class;		/* Class (dev, bjt, etc.) of this device */
-    char *type;		/* Type (name) of this device */
-    Rect *r;		/* Coordinates of 1x1 rectangle entirely inside device */
-    int argc;		/* Size of argv */
-    char *argv[];	/* Tokens for the rest of the dev line.
+efBuildDevice(
+    Def *def,		/* Def to which this connection is to be added */
+    char class,		/* Class (dev, bjt, etc.) of this device */
+    char *type,		/* Type (name) of this device */
+    const Rect *r,	/* Coordinates of 1x1 rectangle entirely inside device */
+    int argc,		/* Size of argv */
+    char *argv[])	/* Tokens for the rest of the dev line.
 			 * Starts with the last two position values, used to
 			 * hash the device record.  The next arguments depend
 			 * on the type of device.  The rest are taken in groups
@@ -892,6 +891,7 @@ efBuildDevice(def, class, type, r, argc, argv)
 		argstart = 2;
 	    break;
 	case DEV_SUBCKT:
+	case DEV_VERILOGA:
 	case DEV_MSUBCKT:
 	case DEV_RSUBCKT:
 	case DEV_CSUBCKT:
@@ -978,6 +978,7 @@ efBuildDevice(def, class, type, r, argc, argv)
 	case DEV_CSUBCKT:
 	case DEV_MSUBCKT:
 	case DEV_SUBCKT:
+	case DEV_VERILOGA:
 	case DEV_DIODE:
 	case DEV_NDIODE:
 	case DEV_PDIODE:
@@ -1308,10 +1309,10 @@ EFGetPortMax(def)
  */
 
 EFNode *
-efBuildDevNode(def, name, isSubsNode)
-    Def *def;
-    char *name;
-    bool isSubsNode;
+efBuildDevNode(
+    Def *def,
+    char *name,
+    bool isSubsNode)
 {
     HashEntry *he;
     EFNodeName *nn;
