@@ -49,9 +49,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
  */
 
 int
-main(argc, argv)
-    int argc;
-    char *argv[];
+main(int argc, char *argv[])
 {
     magicMain(argc, argv);
     exit(0);
@@ -65,4 +63,4 @@ main(argc, argv)
 
 char *MagicVersion = MAGIC_VERSION;
 char *MagicRevision = MAGIC_REVISION;
-char *MagicCompileTime = MAGIC_DATE;
+char *MagicCompileTime = MAGIC_BUILDDATE;
