@@ -25,6 +25,9 @@
 #include <signal.h>
 #include <string.h>
 #include <errno.h>
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
 
 #include "tcltk/tclmagic.h"
 #include "utils/main.h"
@@ -51,7 +54,7 @@
 
 char *MagicVersion = MAGIC_VERSION;
 char *MagicRevision = MAGIC_REVISION;
-char *MagicCompileTime = MAGIC_DATE;
+char *MagicCompileTime = MAGIC_BUILDDATE;
 
 #if TCL_MAJOR_VERSION < 9
 const char *Tclmagic_InitStubsVersion = "8.5";
@@ -366,6 +369,7 @@ _tcl_dispatch(ClientData clientData,
     if (!strncmp(argv0, "::", 2)) argv0 += 2;
 
     objv0 = Tcl_NewStringObj(argv0, strlen(argv0));
+    Tcl_IncrRefCount(objv0); /* this seems needed to ensure it is freed across Tcl_EvalObjv() */
     if (Tcl_GetIndexFromObj(interp, objv0, (const char **)conflicts,
 	"overloaded command", 0, &idx) == TCL_OK)
     {
@@ -393,7 +397,10 @@ _tcl_dispatch(ClientData clientData,
 	Tcl_Free((char *)objv);
 
 	if (result == TCL_OK)
+	{
+	    Tcl_DecrRefCount(objv0);
 	    return result;
+        }
 
 	/* The rule is to execute Magic commands for any Tcl command 	*/
 	/* with the same name that returns an error.  However, this	*/
@@ -417,11 +424,17 @@ _tcl_dispatch(ClientData clientData,
 		    slashptr++;
 
 		if ((dotptr = strrchr(slashptr, '.')) != NULL)
+		{
 		    if (strcmp(dotptr + 1, "mag") && strcmp(dotptr + 1, "gz"))
+		    {
+			Tcl_DecrRefCount(objv0);
 			return result;
+                    }
+                }
 	    }
 	}
     }
+    Tcl_DecrRefCount(objv0);
     Tcl_ResetResult(interp);
 
     if (TxInputRedirect == TX_INPUT_REDIRECTED)
@@ -530,6 +543,61 @@ MakeWindowCommand(char *wname, MagWindow *mw)
     freeMagic(tclcmdstr);
 }
 
+#ifdef HAVE_SETRLIMIT
+static int
+process_rlimit_nofile_ensure(rlim_t nofile)
+{
+    struct rlimit rlim;
+    int err = getrlimit(RLIMIT_NOFILE, &rlim);
+    if (err < 0)
+	return err;
+    rlim_t rlim_cur = rlim.rlim_cur;
+    /* nofile != RLIM_INFINITY && rlim.rlim_max != RLIM_INFINITY */
+    if (nofile > rlim.rlim_max && nofile != rlim.rlim_max)
+	return -1;
+    if (rlim.rlim_cur < nofile || nofile == RLIM_INFINITY)
+    {
+	rlim.rlim_cur = nofile;
+	err = setrlimit(RLIMIT_NOFILE, &rlim);
+    }
+    if (err != 0)
+	TxPrintf("WARNING: process_rlimit_nofile_ensure(%lu) = %d (%d) [rlim_cur=%lu rlim_max=%lu]\n", nofile, err, errno, rlim_cur, rlim.rlim_max);
+    return err;
+}
+#endif /* HAVE_SETRLIMIT */
+
+/* this function encapsulates the default policy on startup */
+static int
+process_rlimit_startup_check(void)
+{
+#ifdef HAVE_GETRLIMIT
+ #if TCL_MAJOR_VERSION < 9
+    /* TCL8 has select() support and no support for poll/epoll for the main event loop */
+    struct rlimit rlim;
+    int err = getrlimit(RLIMIT_NOFILE, &rlim);
+    if (err < 0)
+	return err;
+    if (rlim.rlim_cur > FD_SETSIZE)
+    {
+	TxPrintf("WARNING: RLIMIT_NOFILE is above %d and Tcl_Version<9 this may cause runtime issues [rlim_cur=%lu]\n", FD_SETSIZE, rlim.rlim_cur);
+	return -1;
+    }
+    return 0;
+ #else
+  #ifdef HAVE_SETRLIMIT
+    /* TCL9 has poll/epoll support for the main event loop,
+     * ifdef due to rlim_t type availbility
+     */
+    return process_rlimit_nofile_ensure(4096);
+  #else
+    return -1;
+  #endif /* HAVE_SETRLIMIT */
+ #endif /* TCL_MAJOR_VERSION < 9 */
+#else
+    return -1;
+#endif /* HAVE_GETRLIMIT */
+}
+
 /*------------------------------------------------------*/
 /* Main startup procedure				*/
 /*------------------------------------------------------*/
@@ -588,6 +656,8 @@ _magic_initialize(ClientData clientData,
 	TxPrintf("Using the terminal as the console.\n");
     TxFlushOut();
 
+    process_rlimit_startup_check();
+
     if (mainInitAfterArgs() != 0) goto magicfatal;
 
     /* Registration of commands is performed after calling the	*/
